@@ -1,4 +1,11 @@
-#include "libxslt/libxslt.h"
+#define IN_LIBEXSLT
+#include "libexslt/libexslt.h"
+
+#if defined(WIN32) && !defined (__CYGWIN__)
+#include <win32config.h>
+#else
+#include "config.h"
+#endif
 
 #include <string.h>
 
@@ -305,6 +312,7 @@ exsltFuncFunctionComp (xsltStylesheetPtr style, xmlNodePtr inst) {
     if ((style == NULL) || (inst == NULL))
 	return;
 
+
     {
 	xmlChar *qname;
 
